(this.webpackJsonpstreamlit_folium=this.webpackJsonpstreamlit_folium||[]).push([[0],{6:function(t,a,e){"use strict";e.r(a);var n=e(3),l=e(5),_=null;n.a.events.addEventListener(n.a.RENDER_EVENT,(function(t){var a=t.detail,e=a.args.fig,i=a.args.height,c=a.args.width,o=Object(l.a)(r,250);function r(){var t=__GLOBAL_DATA__,a=t.map.getBounds();n.a.setComponentValue({last_clicked:t.lat_lng_clicked,last_object_clicked:t.last_object_clicked,all_drawings:t.all_drawings,last_active_drawing:t.last_active_drawing,bounds:a})}function s(t){var a=__GLOBAL_DATA__;if(a.last_object_clicked=t.latlng,t.layer.toGeoJSON&&(a.last_active_drawing=t.layer.toGeoJSON()),t.target._layers){var e=[],n=t.target._layers;for(var l in n){var _=n[l];_.toGeoJSON&&e.push(_.toGeoJSON())}a.all_drawings=e}o()}if(null==_)try{_=__GLOBAL_DATA__.map}catch(A){var d=document.getElementById("map_div"),g=document.getElementById("map_div2");if(g&&(g.style.height="".concat(i,"px"),g.style.width="".concat(c,"px")),d){d.style.height="".concat(i,"px"),d.style.width="".concat(c,"px");var u=document.createElement("script"),m=e+"\n          window.__GLOBAL_DATA__ = {\n            map: map_div,\n            bounds: map_div.getBounds(),\n            lat_lng_clicked: null,\n            last_object_clicked: null,\n            all_drawings: null,\n            last_active_drawing: null,\n        };";u.innerHTML=m,document.body.appendChild(u);var p=__GLOBAL_DATA__.map;for(var v in p.on("click",(function(t){__GLOBAL_DATA__.lat_lng_clicked=t.latlng,o()})),p.on("moveend",(function(t){o()})),p._layers){p._layers[v].on("click",s)}p.on("draw:created",(function(t){return s(t)})),n.a.setFrameHeight(),r()}}})),n.a.setComponentReady(),n.a.setFrameHeight()}},[[6,1,2]]]);
//# sourceMappingURL=main.038cacc7.chunk.js.map